# Fluid density with the value to be defined by the user
   set den 1000.0

# Fluid 'horizontal' viscosity with the value to be defined by the user
   set vis 0.14

# Fluid 'vertical' viscosity with the value to be defined by the user
   set vsz 14.0

proc calcOx { } {
     # Read the index of the current node
     set inode [TdynTcl_Index 0]
     set Vey [TdynTcl_VecVal vy $inode]
     return [expr {2*7.292e-05*sin(45*(3.1415926/180.0))*$Vey}]
}

proc calcOy { } {
     set inode [TdynTcl_Index 0]  
     set Vex [TdynTcl_VecVal vx $inode]
     return [expr {-2*7.292e-05*sin(45*(3.1415926/180.0))*$Vex}]
}

# Computation of the surface wind stress

proc TdynTcl_InitiateProblem { } {
    global sstress bstress visc staux stauy btaux btauy vis vsz

    TdynTcl_Message "Executing script imposing stress on water surface and bottom" notice
    # Value of wind stress
    # Tau = Coef_arrastre * Rho_aire/Rho_agua * (Velocidad_viento)^2
    # Tau = 0.0014 * 1.3/1000 * (30 m/s)^2
    set staux 0.0
    set stauy 0.001638
    set btaux 0.000497802
    set btauy 0.000287420

    # Read the indexes of the nodes of the fluid body wind
    set nodes [TdynTcl_GetFluidBodyNodes wind]
    # Create a vector (surface stress)
    set nnode [TdynTcl_NNode 1]
    set tract [::mather::mkvector $nnode 0.0]
    foreach inode $nodes {
	set jnode [TdynTcl_GlobalToFluid $inode]
	::mather::setelem $tract $jnode 1.0
    }
    # Compute the FEM integral
    set sstress [::mather::vmexpr temp=fwind*$tract]
    # Delete the vector created previously
    ::mather::delete $tract

    # Read the indexes of the nodes of the fluid body bottom
    set nodes [TdynTcl_GetFluidBodyNodes bottom]
    # Create a vector (surface stress)
    set nnode [TdynTcl_NNode 1]
    set tract [::mather::mkvector $nnode 0.0]
    foreach inode $nodes {
	set jnode [TdynTcl_GlobalToFluid $inode]
	::mather::setelem $tract $jnode 1.0
    }
    # Compute the FEM integral
    set bstress [::mather::vmexpr temp=fbottom*$tract]
    # Delete the vector created previously
    ::mather::delete $tract

    # Compute the nodal dynamic vector viscosity Mu = Upsilon * Rho_agua
    # vval = Kv - Kh = 14.0 - 1.0e+6 = -999986.0 kg/(m.s)
    set vval [expr $vsz-$vis]
    set nnode [TdynTcl_NNode 1]
    set visc [::mather::mkvector $nnode $vval]
}

# Apply the x component of the wind stress

proc TdynTcl_AssembleFluidMomentumX { } {
    global sstress bstress visc staux stauy btaux btauy

    # Assemble the stress terms
    set nnode [TdynTcl_NNode 1]
    for {set inode 1} {$inode <= $nnode} {incr inode} {
	set jnode [TdynTcl_FluidToGlobal $inode]
	set ri [TdynTcl_GetRhs $jnode]
	set tsi [expr [::mather::getelem $sstress $inode]*$staux]
	set tbi [expr [::mather::getelem $bstress $inode]*$btaux]
	TdynTcl_SetRhs $jnode [expr $ri+$tsi-$tbi]
    }
    # Assemble the viscosity terms
    ::mather::matrix_vector_mult_add fvsys $visc fdnz_dnz 1 
}

# Apply the y component of the wind stress

proc TdynTcl_AssembleFluidMomentumY { } {
    global sstress bstress visc staux stauy btaux btauy
  
    # Assemble the stress terms
    set nnode [TdynTcl_NNode 1]
    for {set inode 1} {$inode <= $nnode} {incr inode} {
	set jnode [TdynTcl_FluidToGlobal $inode]
	set ri [TdynTcl_GetRhs $jnode]
	set tsi [expr [::mather::getelem $sstress $inode]*$stauy]
	set tbi [expr [::mather::getelem $bstress $inode]*$btauy]
	TdynTcl_SetRhs $jnode [expr $ri+$tsi-$tbi]
    }
    # Assemble the viscosity terms
    ::mather::matrix_vector_mult_add fvsys $visc fdnz_dnz 1 
}

# Compute the constant turbulent diffusivity coefficient

proc TdynTcl_AssembleFluidMomentumZ { } {
    global temp visc

    # Assemble the viscosity terms
    ::mather::matrix_vector_mult_add fvsys $visc fdnz_dnz 1 
}